@@ -30,6 +30,7 @@ class Request:
     def __init__(self, token, configuration):
         self.token = token
         self.configuration = configuration
+        self.__refresh_token = None
 
     @property
     def base_url(self):
@@ -38,6 +39,13 @@ class Request:
             port=self.configuration.gateway_port
         )
 
+    @property
+    def auth_url(self):
+        return 'https://{address}:{port}/rest/auth'.format(
+            address=self.configuration.gateway_address,
+            port=self.configuration.gateway_port
+        )
+
     @property
     def headers(self):
         return {'content-type': 'application/json'}
@@ -54,7 +62,7 @@ class Request:
 
     def send_get_request(self, url, **url_params):
         request_url = self.base_url + url.format(**url_params)
-        self._login()
+        version = self.login()
         r = requests.get(request_url,
                          auth=(
                              self.configuration.username,
@@ -62,14 +70,15 @@ class Request:
                          ),
                          verify=self.verify_certificate,
                          timeout=self.configuration.timeout)
-        self._logout()
+
+        self.logout(version)
         response = r.json()
         return r, response
 
     def send_post_request(self, url, params=None, **url_params):
         if params is None:
             params = dict()
-        self._login()
+        version = self.login()
         request_url = self.base_url + url.format(**url_params)
         r = requests.post(request_url,
                           auth=(
@@ -81,14 +90,14 @@ class Request:
                           verify=self.verify_certificate,
                           timeout=self.configuration.timeout)
         response = r.json()
-        self._logout()
+        self.logout(version)
         return r, response
 
     def send_mdm_cluster_post_request(self, url, params=None, **url_params):
         if params is None:
             params = dict()
         response = None
-        self._login()
+        version = self.login()
         request_url = self.base_url + url.format(**url_params)
         r = requests.post(request_url,
                           auth=(
@@ -102,9 +111,77 @@ class Request:
 
         if r.content != b'':
             response = r.json()
-        self._logout()
+        self.logout(version)
         return r, response
 
+    # To perform login based on the API version
+    def login(self):
+        version = self.get_api_version()
+        if utils.check_version(version=version):
+            self._appliance_login()
+        else:
+            self._login()
+        return version
+
+    # To perform logout based on the API version
+    def logout(self, version):
+        if utils.check_version(version=version):
+            self._appliance_logout()
+        else:
+            self._logout()
+
+    # Get the Current API version
+    def get_api_version(self):
+        request_url = self.base_url + '/version'
+        self._login()
+        r = requests.get(request_url,
+                         auth=(
+                             self.configuration.username,
+                             self.token.get()),
+                         verify=self.verify_certificate,
+                         timeout=self.configuration.timeout)
+        response = r.json()
+        return response
+
+    # API Login method for 4.0 and above.
+    def _appliance_login(self):
+        request_url = self.auth_url + '/login'
+        payload = {"username": "%s" % self.configuration.username,
+                   "password": "%s" % self.configuration.password
+                   }
+        r = requests.post(request_url, headers=self.headers, json=payload,
+                          verify=self.verify_certificate,
+                          timeout=self.configuration.timeout
+                          )
+        if r.status_code != requests.codes.ok:
+            exc = exceptions.PowerFlexFailQuerying('token')
+            LOG.error(exc.message)
+            raise exc
+        response = r.json()
+        token = response['access_token']
+        self.token.set(token)
+        self.__refresh_token = response['refresh_token']
+
+    # API logout method for 4.0 and above.
+    def _appliance_logout(self):
+        request_url = self.auth_url + '/logout'
+        token = self.token.get()
+        headers = {'Authorization': 'Bearer {0}'.format(token),
+                   'content-type': 'application/json'
+                   }
+        data = {'refresh_token': '{0}'.format(self.__refresh_token)}
+        r = requests.post(request_url, headers=headers, json=data,
+                          verify=self.verify_certificate,
+                          timeout=self.configuration.timeout
+                          )
+
+        if r.status_code != requests.codes.no_content:
+            exc = exceptions.PowerFlexFailQuerying('token')
+            LOG.error(exc.message)
+            raise exc
+        self.token.set("")
+        self.__refresh_token = None
+
     def _login(self):
         request_url = self.base_url + '/login'
 
@@ -148,6 +225,7 @@ class EntityRequest(Request):
     base_relationship_url = base_entity_url + '/relationships/{related}'
     base_object_url = '/instances/{entity}/action/{action}'
     query_mdm_cluster_url = '/instances/{entity}/queryMdmCluster'
+    list_statistics_url = '/types/{entity}/instances/action/{action}'
     entity_name = None
 
     @property
