@@ -23,6 +23,7 @@ from PyPowerFlex.objects.storage_pool import StoragePool
 from PyPowerFlex.objects.acceleration_pool import AccelerationPool
 from PyPowerFlex.objects.system import System
 from PyPowerFlex.objects.volume import Volume
+from PyPowerFlex.objects.utility import PowerFlexUtility
 
 
 __all__ = [
@@ -36,4 +37,5 @@ __all__ = [
     'AccelerationPool',
     'System',
     'Volume',
+    'PowerFlexUtility'
 ]
